// DaemonCSM.cpp: implementation of the DaemonCSM class.
//
//////////////////////////////////////////////////////////////////////

#include "include.h"
#include "ConfigData.h"
#include "ConfigLoader.h"
#include "TmrSMS.h"
#include "ConfigData.h"
#include "IpcMsg.h"
#include "DaemonCSM.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int DaemonCSM::minus11=-11;
CSM_DESC DaemonCSM::csm_desc=
{
	(int (*)(CSM_RAM*, GBS_MSG*))DaemonCSM::OnMessage,
	(void (*)(CSM_RAM*))DaemonCSM::OnCreate,
#ifdef	NEWSGOLD
	0,0,0,0,
#endif
	(void (*)(CSM_RAM*))DaemonCSM::OnClose,
	sizeof(DAEMON_CSM),
	1,
	&DaemonCSM::minus11
};

DaemonCSM::DaemonCSM()
{
	memcpy(&this->daemon_csm_desc.csm_desc, &DaemonCSM::csm_desc, sizeof(CSM_DESC));
	this->csm_name=CreateLocalWS(&this->daemon_csm_desc.csm_name, this->csm_name_body, CSM_NAME_LEN);
	this->csm_name->ws_malloc=NAMECSM_MAGIC1;
	this->csm_name->ws_mfree=NAMECSM_MAGIC2;
	this->ws_text=CreateLocalWS(&this->_ws_text, this->ws_text_body, TEXT_MSG_LEN);
}

DaemonCSM::~DaemonCSM()
{

}

void DaemonCSM::OnCreate(DAEMON_CSM *daemon_csm)
{
	int t_len;
	TTime time;
	TDate date;
	int t_res;
	int sec1;
	int sec2;
	ConfigLoader::Init();
	IpcMsg::InitIpcName();
	if(strlen(ConfigData::Number))
	{
		strcpy(daemon_csm->daemon->number, ConfigData::Number);
	}
	else
	{
		ShowMSG(1,(int)"Error: Number!");
		daemon_csm->csm_ram.state=-3;
		return;
	}
	if((t_len=strlen(ConfigData::CfgText)))
	{
		utf8_2ws(daemon_csm->daemon->ws_text, ConfigData::CfgText, t_len);
	}
	else
	{
		ShowMSG(1,(int)"Error: Text!");
		daemon_csm->csm_ram.state=-3;
		return;
	}
	if(ConfigData::Directly)
	{
		SendTmrProc();
		return;
	}
	GetDateTime(&date, &time);
	t_res=CmpDates((TDate *)&ConfigData::CfgDate, &date);
	if(t_res<0)
	{
		ShowMSG(1,(int)"Error: Date!");
		daemon_csm->csm_ram.state=-3;
		return;
	}
	if(t_res==0)
	{
		if(CmpTimes((TTime *)&ConfigData::CfgTime, &time)<=0)
		{
			ShowMSG(1,(int)"Error: Time!");
			daemon_csm->csm_ram.state=-3;
			return;
		}
	}
	//GetSecondsFromDateTimeSince1997(&sec1, &ConfigData::CfgDate, &ConfigData::CfgTime);
	//GetSecondsFromDateTimeSince1997(&sec2, &date, &time);
	if(
		!GetSecondsFromDateTime(&sec1, (TDate *)&ConfigData::CfgDate, (TTime *)&ConfigData::CfgTime, &date)
		|| !GetSecondsFromDateTime(&sec2, &date, &time, &date)
		|| (t_res=sec1-sec2)<=0
		)
	{
		ShowMSG(1,(int)"Error: Unk!");
		daemon_csm->csm_ram.state=-3;
		return;
	}
	daemon_csm->daemon->UpdateCSMname();
	GBS_StartTimerProc(&daemon_csm->daemon->send_tmr, t_res*1300, SendTmrProc);
}

void DaemonCSM::OnClose(DAEMON_CSM *daemon_csm)
{
	GBS_DelTimer(&daemon_csm->daemon->send_tmr);
	delete daemon_csm->daemon;
	SUBPROC((void *)TmrSMS::Killer);
}

int DaemonCSM::OnMessage(DAEMON_CSM *daemon_csm, GBS_MSG *gbs_msg)
{
	if(gbs_msg->msg==MSG_IPC)
	{
		IPC_REQ *ipc;
		if(
			(ipc=(IPC_REQ *)gbs_msg->data0)!=0
			&& strcmp(ipc->name_to, IpcMsg::ipc_name)==0
			)
		{
			switch(gbs_msg->submess)
			{
			case IPC_MSG_SEND:
				SUBPROC((void *)SendSMS,
					daemon_csm->daemon->number,
					daemon_csm->daemon->ws_text);
				break;
			case IPC_MSG_CLOSE:
				daemon_csm->csm_ram.state=-3;
				break;
			}
		}
	}
	return 1;
}

int DaemonCSM::Create()
{
	int csm_id;
	DAEMON_CSM daemon_csm;
	CSM_RAM *save_cmpc;
	CSMROOT *csmr=CSM_root();
	zeromem(&daemon_csm, sizeof(DAEMON_CSM));
	daemon_csm.daemon=this;
	LockSched();
	save_cmpc=csmr->csm_q->current_msg_processing_csm;
	csmr->csm_q->current_msg_processing_csm=(CSM_RAM *)csmr->csm_q->csm.first;
	csm_id=CreateCSM(&this->daemon_csm_desc.csm_desc,&daemon_csm,0);
	csmr->csm_q->current_msg_processing_csm=save_cmpc;
	UnlockSched();
	return csm_id;
}

void DaemonCSM::UpdateCSMname()
{
	wsprintf(this->csm_name, "TmrSMS %s %04d.%02d.%02d %02d:%02d:%02d",
		ConfigData::Number,
		ConfigData::CfgDate.year,
		ConfigData::CfgDate.month,
		ConfigData::CfgDate.day,
		ConfigData::CfgTime.hour,
		ConfigData::CfgTime.min,
		ConfigData::CfgTime.sec
		);
}

void DaemonCSM::SendTmrProc()
{
	IpcMsg::Send(IPC_MSG_SEND);
}

void DaemonCSM::SendSMS(const char *number, WSHDR *text)
{
	int wlen;
	if(number!=NULL
		&& text!=NULL
		&& strlen(number)!=0
		&& (wlen=wstrlen(text))!=0
		)
	{
		WSHDR *ws=AllocWS(wlen);
		wstrcpy(ws, text);
		::SendSMS(ws, number, MMI_CEPID, MSG_SMS_RX-1, 6);
	}
	IpcMsg::Send(IPC_MSG_CLOSE);
}
